/**
 * Forces the loading of the real number of messages (or more)
 * from the setting `archived_messages_page_size`.
 */
converse.plugins.add('sib-mam-history', {
  dependencies: [
    'converse-mam',
  ],
  initialize() {
    const { api } = this._converse;
    let counter = 0;

    api.listen.on('MAMResult', async data => {

      const max = +api.settings.get('archived_messages_page_size');
      const messages = await Promise.all(data.messages);

      // Increase counter with the messages that contains a body
      counter += messages.filter(attrs => attrs.body).length;

      // Stop if the max value is not specified
      // Stop if there are less messages available than the page size
      // Stop if the counter registers more messages than the max value
      if (!max || messages.length < max || counter >= max) {
        counter = 0;
        return;
      }

      const is_groupchat = data.chatbox.get('type') === converse.CHATROOMS_TYPE;
      const oldest_message = messages.pop();

      if (oldest_message) {
        const by_jid = is_groupchat ? data.chatbox.get('jid') : converse.bare_jid;
        const stanza_id = oldest_message && oldest_message['stanza_id '.concat(by_jid)];

        if (stanza_id) {
          await data.chatbox.fetchArchivedMessages({
            'before': stanza_id,
          });
        } else {
          await data.chatbox.fetchArchivedMessages({
            'end': oldest_message['time'],
          });
        }
      }
    });
  },
});
